/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: test.c,v 1.7 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#define _GNU_SOURCE /* For posix_memalign.  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <errno.h>

#include "test.h"
#include "zoid_api.h"

static void buffer_cb(void* userbuf, void* priv)
{
    fprintf(stderr, "buffer_cb invoked on %d with userbuf %p, priv %p\n",
	    __zoid_calling_process_id(), userbuf, priv);
    free(userbuf);
}

int test_read(char* buf /*out:arr:size=+1:zerocopy:userbuf */,
	      size_t *count /* inout:ptr */)
{
    int i;

    fprintf(stderr, "test_read invoked on %d with buf %p, count %d\n",
	    __zoid_calling_process_id(), buf, *count);

    if (posix_memalign((void*)&buf, 16, *count))
    {
	*count = 0;
	return -ENOMEM;
    }
    fprintf(stderr, "%d: allocated buffer %p\n",
	    __zoid_calling_process_id(), buf);

    __zoid_register_userbuf(buf, &buffer_cb, NULL);

    for (i = 0; i < *count; i++)
	buf[i] = random();

    return *count;
}

int test_read_noub(char* buf /*out:arr:size=+1:zerocopy */,
		   size_t *count /* inout:ptr */)
{
    int i;

    fprintf(stderr, "test_read_noub invoked on %d with buf %p, count %d\n",
	    __zoid_calling_process_id(), buf, *count);

    for (i = 0; i < *count; i++)
	buf[i] = random();

    return *count;
}

void*
test_write_allocate_cb(int len)
{
    void* ptr;

    if (posix_memalign(&ptr, 16, len + 4))
	ptr = NULL;
    memcpy(ptr + len, "abcd", 4);

    fprintf(stderr, "test_write_allocate_cb invoked on %d with len %d, "
	    "returning %p\n", __zoid_calling_process_id(), len, ptr);

    return ptr;
}

int test_write(const void *buf /* in:arr:size=+1:zerocopy:userbuf */,
	       size_t count /* in:obj */)
{
    int i;

    fprintf(stderr, "test_write invoked on %d with buf %p, count %d\n",
	    __zoid_calling_process_id(), buf, count);
#if 1
    for (i = 0; i < count; i++)
    {
	char ret;
	if ((ret = random()) != ((char*)buf)[i])
	{
	    fprintf(stderr, "verification failed at offset %d: %d != %d\n",
		    i, ret, ((char*)buf)[i]);
	    free((void*)buf);
	    return -1;
	}
    }
#endif
    if (memcmp(buf + count, "abcd", 4))
    {
	fprintf(stderr, "end-of-buffer verification failed!\n");
	free((void*)buf);
	return -1;
    }

    free((void*)buf);

    return count;
}

int test_write_noub(const void *buf /* in:arr:size=+1:zerocopy */,
		    size_t count /* in:obj */)
{
    int i;

    fprintf(stderr, "test_write_noub invoked on %d with buf %p, count %d\n",
	    __zoid_calling_process_id(), buf, count);

    for (i = 0; i < count; i++)
    {
	char ret;
	if ((ret = random()) != ((char*)buf)[i])
	{
	    fprintf(stderr, "verification failed at offset %d: %d != %d\n",
		    i, ret, ((char*)buf)[i]);
	    return -1;
	}
    }

    return count;
}
