/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: link.c,v 1.5 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#include "unix.h"

#include <stdio.h>

#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include "unix_internal.h"

int unix_link(const char *oldpath /* in:str */,
	      const char *newpath /* in:str */) /* shortout */
{
    int ret;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);

    if ((ret = link(oldpath, newpath)) == -1)
	ret = -errno;

    unix__cwd_mask_unlock();

    return ret;
}

int unix_symlink(const char *oldpath /* in:str */,
		 const char *newpath /* in:str */) /* shortout */
{
    int ret;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);

    if ((ret = symlink(oldpath, newpath)) == -1)
	ret = -errno;

    unix__cwd_mask_unlock();

    return ret;
}

int unix_unlink(const char *pathname /* in:str */) /* shortout */
{
    int ret;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);

    if ((ret = unlink(pathname)) == -1)
	ret = -errno;

    unix__cwd_mask_unlock();

    return ret;
}

int unix_readlink(const char *path /* in:str */,
		  char *buf /* out:arr:size=+1 */,
		  size_t bufsiz /* in:obj */)
{
    int ret;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);

    if ((ret = readlink(path, buf, bufsiz)) == -1)
	ret = -errno;

    unix__cwd_mask_unlock();

    return ret;
}

int unix_rename(const char *oldpath /* in:str */,
		const char *newpath /* in:str */) /* shortout */
{
    int ret;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);

    if ((ret = rename(oldpath, newpath)) == -1)
	ret = -errno;

    unix__cwd_mask_unlock();

    return ret;
}
