/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: open_close.c,v 1.6 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#include "unix.h"

#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include "unix_internal.h"

int unix_open(const char *pathname /* in:str */,
	      int flags /* in:obj */,
	      mode_t mode /* in:obj */) /* shortout */
{
    int fd;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);
    umask(unix__getumask());

    if ((fd = open(pathname, flags, mode)) == -1)
	fd = -errno;

    if (fd >= 0)
	if ((fd = unix__record_fd(fd)) == -1)
	    fd = -errno;

    unix__cwd_mask_unlock();

    return fd;
}

int unix_close(int fd /* in:obj */) /* short */
{
    int server_fd;

    if ((server_fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    unix__clear_client_fd(fd);

    if (server_fd >= 0 && close(server_fd) == -1)
	return -errno;
    else
	return 0;
}

int unix_dup(int oldfd /* in:obj */) /* short */
{
    int newfd;

    if ((oldfd = unix__translate_client_fd(oldfd)) == -1)
	return -errno;

    newfd = (oldfd >= 0 ? dup(oldfd) : oldfd);

    if (newfd != -1)
	newfd = unix__record_fd(newfd);

    if (newfd == -1)
	return -errno;
    else
	return newfd;
}

int unix_dup2(int oldfd /* in:obj */,
	      int newfd /* in:obj */) /* short */
{
    int newfd_server;

    if ((oldfd = unix__translate_client_fd(oldfd)) == -1)
	return -errno;

    if ((newfd_server = unix__translate_client_fd(newfd)) >= 0)
	close(newfd_server);

    newfd_server = (oldfd >= 0 ? dup(oldfd) : oldfd);

    if (newfd_server != -1)
	newfd = unix__record_fd_at(newfd_server, newfd);

    if (newfd == -1)
	return -errno;
    else
	return newfd;
}
