/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: read_write.c,v 1.9 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#define _XOPEN_SOURCE 500
#include "unix.h"

#include <stdio.h>
#include <stdlib.h>

#include <alloca.h>
#include <errno.h>
#include <unistd.h>

#include "zoid_api.h"

#include "unix_internal.h"

ssize_t unix_read(int fd /* in:obj */,
		  void *buf /* out:arr:size=+1 */,
		  size_t *count /* inout:ptr */)
{
    ssize_t ret;

    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if ((ret = read(fd, buf, *count)) == -1)
    {
	*count = 0;
	return -errno;
    }
    else
    {
	*count = ret;
	return ret;
    }
}

ssize_t unix_write(int fd /* in:obj */,
		   const void *buf /* in:arr:size=+1 */,
		   size_t count /* in:obj */) /* shortout */
{
    ssize_t ret;

    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if (fd == FD_DEFAULT_STDOUT || fd == FD_DEFAULT_STDERR)
    {
	if ((ret =__zoid_send_output(__zoid_calling_process_id(),
				     (fd == FD_DEFAULT_STDOUT ? 1 : 2), buf,
				     count)) == -1)
	    return -errno;
	return ret;
    }

    if ((ret = write(fd, buf, count)) == -1)
	return -errno;
    else
	return ret;
}

ssize_t unix_pread(int fd /* in:obj */,
		   void *buf /* out:arr:size=+1 */,
		   size_t *count /* inout:ptr */,
		   off_t offset /* in:obj */) /* shortin */
{
    ssize_t ret;

    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if ((ret = pread(fd, buf, *count, offset)) == -1)
    {
	*count = 0;
	return -errno;
    }
    else
    {
	*count = ret;
	return ret;
    }
}

ssize_t unix_pwrite(int fd /* in:obj */,
		    const void *buf /* in:arr:size=+1 */,
		    size_t count /* in:obj */,
		    off_t offset /* in:obj */) /* shortout */
{
    ssize_t ret;

    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if ((ret = pwrite(fd, buf, count, offset)) == -1)
	return -errno;
    else
	return ret;
}

ssize_t unix_pread64(int fd /* in:obj */,
		     void *buf /* out:arr:size=+1 */,
		     size_t *count /* inout:ptr */,
		     off64_t offset /* in:obj */) /* shortin */
{
    ssize_t ret;

    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if ((ret = pread64(fd, buf, *count, offset)) == -1)
    {
	*count = 0;
	return -errno;
    }
    else
    {
	*count = ret;
	return ret;
    }
}

ssize_t unix_pwrite64(int fd /* in:obj */,
		      const void *buf /* in:arr:size=+1 */,
		      size_t count /* in:obj */,
		      off64_t offset /* in:obj */) /* shortout */
{
    ssize_t ret;

    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if ((ret = pwrite64(fd, buf, count, offset)) == -1)
	return -errno;
    else
	return ret;
}

off_t unix_lseek(int fildes /* in:obj */,
		 off_t offset /* in:obj */,
		 int whence /* in:obj */) /* short */
{
    off_t ret;

    if ((fildes = unix__translate_client_fd(fildes)) == -1)
	return -errno;

    if ((ret = lseek(fildes, offset, whence)) == -1)
	return -errno;
    else
	return ret;
}

off64_t unix_lseek64(int fildes /* in:obj */,
		     off64_t offset /* in:obj */,
		     int whence /* in:obj */) /* short */
{
    off64_t ret;

    if ((fildes = unix__translate_client_fd(fildes)) == -1)
	return -errno;

    if ((ret = lseek64(fildes, offset, whence)) == -1)
	return -errno;
    else
	return ret;
}

ssize_t unix_readv(int fd /* in:obj */,
		   void** bases /* out:arr2d:size=+1 */,
		   size_t* lens /* inout:arr:size=+1 */,
		   int count /* in:obj */)
{
    struct iovec* vectors;
    int i;
    ssize_t ret;

    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    vectors = alloca(sizeof(*vectors) * count);

    for (i = 0; i < count; i++)
    {
	vectors[i].iov_base = bases[i];
	vectors[i].iov_len = lens[i];
    }

    if ((ret = readv(fd, vectors, count)) == -1)
    {
	ret = -errno;
	for (i = 0; i < count; i++)
	    lens[i] = 0;
    }
    else
    {
	int n = ret;

	for (i = 0; i < count; i++)
	{
	    if (n >= lens[i])
		n -= lens[i];
	    else
	    {
		lens[i] = n;
		if (n > 0)
		    n = 0;
	    }
	}
    }

    return ret;
}

ssize_t unix_writev(int fd /* in:obj */,
		    const void** bases /* in:arr2d:size=+1 */,
		    size_t* lens /* in:arr:size=+1 */,
		    int count /* in:obj */) /* shortout */
{
    struct iovec* vectors;
    int i;
    ssize_t ret;

    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if (fd == FD_DEFAULT_STDOUT || fd == FD_DEFAULT_STDERR)
    {
	ret = 0;
	for (i = 0; i < count; i++)
	{
	    int n;
	    if ((n =__zoid_send_output(__zoid_calling_process_id(),
				       (fd == FD_DEFAULT_STDOUT ? 1 : 2),
				       bases[i], lens[i])) == -1)
		return -errno;
	    ret += n;
	}
	return ret;
    }

    vectors = alloca(sizeof(*vectors) * count);

    for (i = 0; i < count; i++)
    {
	vectors[i].iov_base = (void*)bases[i];
	vectors[i].iov_len = lens[i];
    }

    if ((ret = writev(fd, vectors, count)) == -1)
	ret = -errno;

    return ret;
}
