/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: sockets.c,v 1.6 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#include "unix.h"

#include <errno.h>
#include <sys/socket.h>

#include "unix_internal.h"

int unix_socket(int domain /* in:obj */,
		int type /* in:obj */,
		int protocol /* in:obj */) /* short */
{
    int fd;

    fd = socket(domain, type, protocol);

    if (fd != -1)
	fd = unix__record_fd(fd);

    if (fd == -1)
	return -errno;
    else
	return fd;
}

int unix_connect(int sockfd /* in:obj */,
		 void *serv_addr /* in:arr:size=+1 */,
		 socklen_t addrlen /* in:obj */) /* short */
{
    if ((sockfd = unix__translate_client_fd(sockfd)) == -1)
	return -errno;

    if (connect(sockfd, serv_addr, addrlen) == -1)
	return -errno;
    else
	return 0;
}

ssize_t unix_send(int s /* in:obj */,
		  const void *buf /* in:arr:size=+1 */,
		  size_t len /* in:obj */,
		  int flags /* in:obj */) /* shortout */
{
    ssize_t ret;

    if ((s = unix__translate_client_fd(s)) == -1)
	return -errno;

    if ((ret = send(s, buf, len, flags)) == -1)
	return -errno;
    else
	return ret;
}

ssize_t unix_recv(int s /* in:obj */,
		  void *buf /* out:arr:size=+1 */,
		  size_t *len /* inout:ptr */,
		  int flags /* in:obj */) /* shortin */
{
    ssize_t ret;

    if ((s = unix__translate_client_fd(s)) == -1)
	return -errno;

    if ((ret = recv(s, buf, *len, flags)) == -1)
    {
	*len = 0;
	return -errno;
    }
    else
    {
	*len = ret;
	return ret;
    }
}

ssize_t unix_sendto(int s /* in:obj */,
		    const void *buf /* in:arr:size=+1 */,
		    size_t len /* in:obj */,
		    int flags /* in:obj */,
		    const void *to /* in:arr:size=+1:nullok */,
		    socklen_t tolen /* in:obj */) /* shortout */
{
    ssize_t ret;

    if ((s = unix__translate_client_fd(s)) == -1)
	return -errno;

    if ((ret = sendto(s, buf, len, flags, to, tolen)) == -1)
	return -errno;
    else
	return ret;
}

ssize_t unix_recvfrom(int s /* in:obj */,
		      void *buf /* out:arr:size=+1 */,
		      size_t *len /* inout:ptr */,
		      int flags /* in:obj */,
		      void *from /* out:arr:size=+1:nullok */,
		      socklen_t *fromlen /* inout:ptr */) /* shortin */
{
    ssize_t ret;

    if ((s = unix__translate_client_fd(s)) == -1)
	return -errno;

    if ((ret = recvfrom(s, buf, *len, flags, from, fromlen)) == -1)
    {
	*len = 0;
	return -errno;
    }
    else
    {
	*len = ret;
	return ret;
    }
}

int unix_getsockname(int s /* in:obj */,
		     void *name /* out:arr:size=+1 */,
		     socklen_t *namelen /* inout:ptr */) /* shortin */
{
    if ((s = unix__translate_client_fd(s)) == -1)
	return -errno;

    if (getsockname(s, name, namelen) == -1)
	return -errno;
    else
	return 0;
}

int unix_getpeername(int s /* in:obj */,
		     void *name /* out:arr:size=+1 */,
		     socklen_t *namelen /* inout:ptr */) /* shortin */
{
    if ((s = unix__translate_client_fd(s)) == -1)
	return -errno;

    if (getpeername(s, name, namelen) == -1)
	return -errno;
    else
	return 0;
}

int unix_getsockopt(int s /* in:obj */,
		    int level /* in:obj */,
		    int optname /* in:obj */,
		    void *optval /* out:arr:size=+1:nullok */,
		    socklen_t *optlen /* inout:ptr */)
{
    if ((s = unix__translate_client_fd(s)) == -1)
	return -errno;

    if (getsockopt(s, level, optname, optval, optlen) == -1)
	return -errno;
    else
	return 0;
}

int unix_setsockopt(int s /* in:obj */,
		    int level /* in:obj */,
		    int optname /* in:obj */,
		    const void *optval /* in:arr:size=+1:nullok */,
		    socklen_t optlen /* in:obj */)
{
    if ((s = unix__translate_client_fd(s)) == -1)
	return -errno;

    if (setsockopt(s, level, optname, optval, optlen) == -1)
	return -errno;
    else
	return 0;
}
