/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: test10.c,v 1.3 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

/* This test verifies that readv() and writev() based on large chunks
   (including much larger than the default zoid buffer size) work correctly. */

#include <assert.h>
#include <stdio.h>
#include <string.h>

#include <errno.h>
#include <fcntl.h>
#include <sys/uio.h>
#include <unistd.h>

char buffer[20*1024*1024+2048];

int main(void)
{
    int fd;
    int i;

    fd = open("/dev/zero", O_RDWR);
    assert(fd != -1);

    fprintf(stderr, "performing write test...\n");

    for (i = 1024 * 1024; i < sizeof(buffer) / 2; i += 1024 * 1024)
    {
	int n;
	struct iovec iov[2];

	iov[0].iov_base = buffer;
	iov[0].iov_len = i;
	iov[1].iov_base = buffer + i;
	iov[1].iov_len = i;

	if ((n = writev(fd, iov, 2)) != i * 2)
	{
	    fprintf(stderr, "writing %d bytes returned %d, errno %s\n",
		    i * 2, n, strerror(errno));
	    break;
	}
    }

    fprintf(stderr, "performing read test...\n");

    for (i = 1024 * 1024; i < sizeof(buffer) / 2; i += 1024 * 1024)
    {
	int n;
	struct iovec iov[2];

	iov[0].iov_base = buffer;
	iov[0].iov_len = i;
	iov[1].iov_base = buffer + i;
	iov[1].iov_len = i;

	if ((n = readv(fd, iov, 2)) != i * 2)
	{
	    fprintf(stderr, "reading %d bytes returned %d, errno %s\n",
		    i * 2, n, strerror(errno));
	    break;
	}
    }

    fprintf(stderr, "all done\n");

    return 0;
}
