/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: test11.c,v 1.4 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

/* This test verifies that reading and writing at the same time (from
   multiple CN processes) works correctly.  */

#define _LARGEFILE_SOURCE
#define _LARGEFILE64_SOURCE
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include <mpi.h>
#include <rts.h>

#define FILENAME "/dev/zero"
#define INIT_BUF 131072
#define BUF_INCREASE 240
#define COUNT 5000

int
main(int argc, char* argv[])
{
    int buf_size;
    int fd;
    int i;
    char* buffer;
    int size, rank;

    MPI_Init(&argc, &argv);
    MPI_Comm_size(MPI_COMM_WORLD, &size);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);

    fd = open(FILENAME, O_RDWR | O_CREAT | O_TRUNC | O_LARGEFILE,
	      0666);
    if (fd == -1)
    {
	perror("File open for writing");
	return 1;
    }

    buffer = malloc(INIT_BUF + BUF_INCREASE);
    assert(buffer);

    if (rts_get_processor_id() == 0)
    {
	printf("%d: doing writing\n", rank);
	fflush(stdout);

	for (buf_size = INIT_BUF; buf_size < INIT_BUF + BUF_INCREASE;
	     buf_size++)
	{
	    MPI_Barrier(MPI_COMM_WORLD);

	    for (i = 0; i < COUNT; i++)
	    {
		int n;
		if ((n = write(fd, buffer, buf_size)) != buf_size)
		{
		    fprintf(stderr, "Short write: %d (errno %d, i %d)\n", n, errno, i);
		    return 1;
		}
	    }

	    printf("%d: write %d finished\n", rank, buf_size);
	    fflush(stdout);
	}
    }
    else
    {
	printf("%d: doing reading\n", rank);
	fflush(stdout);

	for (buf_size = INIT_BUF; buf_size < INIT_BUF + BUF_INCREASE;
	     buf_size++)
	{
	    MPI_Barrier(MPI_COMM_WORLD);

	    for (i = 0; i < COUNT; i++)
	    {
		int n;
		if ((n = read(fd, buffer, buf_size)) != buf_size)
		{
		    fprintf(stderr, "Short read: %d (errno %d, i %d)\n", n, errno, i);
		    return 1;
		}
	    }

	    printf("%d: read %d finished\n", rank, buf_size);
	    fflush(stdout);
	}
    }

    close(fd);

    MPI_Barrier(MPI_COMM_WORLD);

    MPI_Finalize();

    return 0;
}
