/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: test4.c,v 1.3 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

/* This test verifies that reading from a file in small chunks (1-1024 bytes)
   using readv() works correctly.  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <fcntl.h>
#include <sys/uio.h>
#include <unistd.h>

char buf1[1024*1024] __attribute__((aligned(16)));
char buf2[1026*1024] __attribute__((aligned(16)));

int main(void)
{
    int fd;
    int i, size;
    struct iovec iov[2];
    int mismatch_cnt = 0;

    if (!(fd = open("test3.file", O_CREAT | O_RDWR | O_TRUNC, 0666)))
    {
	perror("open");
	return 1;
    }

    for (i = 0; i < sizeof(buf1); i++)
	buf1[i] = random();

    iov[0].iov_base = buf1;
    iov[0].iov_len = sizeof(buf1);
    if (writev(fd, iov, 1) != sizeof(buf1))
    {
	perror("short write");
	return 1;
    }

    for (size = 1; size < 1024; size++)
    {
	lseek(fd, 0, SEEK_SET);
	memset(buf2, 0, sizeof(buf2));
	for (i = 0; i < 1024; i += 2)
	{
	    int j;
	    int n;
	    iov[0].iov_base = buf2 + i * size;
	    iov[0].iov_len = size;
	    iov[1].iov_base = buf2 + (i + 1) * size;
	    iov[1].iov_len = size;
	    if ((n = readv(fd, iov, 2)) != size * 2)
	    {
		perror("readv");
		fprintf(stderr, "size %d, i %d, n %d\n", size, i, n);
		return 1;
	    }
	    for (j = (i + 2) * size; j < (i + 3) * size; j++)
		if (buf2[j])
		{
		    fprintf(stderr, "Non-zero byte past read area!\n");
		    return 1;
		}
	}
	for (i = 0; i < 1024 * size; i++)
	    if (buf1[i] != buf2[i])
	    {
		fprintf(stderr, "Read mismatch!\n");
		fprintf(stderr, "size %d, i %d, buf1 0x%x, buf2 0x%x\n",
			size, i, buf1[i], buf2[i]);
		if (++mismatch_cnt > 250)
		    return 1;
	    }
	for (i = 1024 * size; i < sizeof(buf2); i++)
	    if (buf2[i])
	    {
		fprintf(stderr, "Non-zero byte past read area!\n");
		return 1;
	    }
	printf("Successfully tested with size %d\n", size);
    }

    close(fd);

    return 0;
}
