/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: varia.c,v 1.6 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#include "unix.h"

#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/file.h>

#include "unix_internal.h"

mode_t unix_umask(mode_t mask /* in:obj */) /* short */
{
    return unix__setumask(mask);
}

int unix_truncate(const char *path /* in:str */,
		  off_t length /* in:obj */) /* shortout */
{
    int ret;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);

    if ((ret = truncate(path, length)) == -1)
	ret = -errno;

    unix__cwd_mask_unlock();

    return ret;
}

int unix_truncate64(const char *path /* in:str */,
		    off64_t length /* in:obj */) /* shortout */
{
    int ret;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);

    if ((ret = truncate64(path, length)) == -1)
	ret = -errno;

    unix__cwd_mask_unlock();

    return ret;
}

int unix_ftruncate(int fd /* in:obj */,
		   off_t length /* in:obj */) /* short */
{
    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if (ftruncate(fd, length) == -1)
	return -errno;
    else
	return 0;
}

int unix_ftruncate64(int fd /* in:obj */,
		     off64_t length /* in:obj */) /* short */
{
    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if (ftruncate64(fd, length) == -1)
	return -errno;
    else
	return 0;
}

int unix_fsync(int fd /* in:obj */) /* short */
{
    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if (fsync(fd) == -1)
	return -errno;
    else
	return 0;
}

int unix_fcntl(int fd /* in:obj */,
	       int cmd /* in:obj */,
	       void *arg /* inout:arr:size=+1:nullok */,
	       int *arglen /* inout:ptr */)
{
    int ret;

    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    switch (cmd)
    {
	case F_GETFL:
	    ret = fcntl(fd, cmd);
	    break;

	case F_SETFL:
	case F_DUPFD:
	    *arglen = 0;
	    ret = fcntl(fd, cmd, *(long*)arg);
	    break;

	case F_SETLK:
	case F_SETLKW:
	case F_SETLK64:
	case F_SETLKW64:
	    *arglen = 0;
	    /* fall through */
	case F_GETLK:
	case F_GETLK64:
	    /* F_GETLK* are the only ones that modify their arg.  For the rest
	       we set arglen to 0 to prevent the (unmodified) argument from
	       being returned to the caller.  */
	    ret = fcntl(fd, cmd, arg);
	    break;

	default:
	    *arglen = 0;
	    return -EINVAL;
    }

    if (ret == -1)
	return -errno;
    else
	return ret;
}

int unix_statfs(const char *file /* in:str */,
		struct statfs *buf /* out:ptr */)
{
    int ret;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);

    if ((ret = statfs(file, buf)) == -1)
	ret = -errno;

    unix__cwd_mask_unlock();

    return ret;
}

int unix_fstatfs(int fd /* in:obj */,
		 struct statfs *buf /* out:ptr */)
{
    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if (fstatfs(fd, buf) == -1)
	return -errno;
    else
	return 0;
}

int unix_statfs64(const char *file /* in:str */,
		  struct statfs64 *buf /* out:ptr */)
{
    int ret;
    int cwd;

    unix__cwd_mask_lock();

    if ((cwd = unix__getcwd_fd()) != -1)
	fchdir(cwd);

    if ((ret = statfs64(file, buf)) == -1)
	ret = -errno;

    unix__cwd_mask_unlock();

    return ret;
}

int unix_fstatfs64(int fd /* in:obj */,
		   struct statfs64 *buf /* out:ptr */)
{
    if ((fd = unix__translate_client_fd(fd)) == -1)
	return -errno;

    if (fstatfs64(fd, buf) == -1)
	return -errno;
    else
	return 0;
}
