/* Please note that this file is shared between ZOID and GLIBC!  */

/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: unix.h,v 1.13 2007/06/27 15:10:45 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_ORIG
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#ifndef UNIX_H
#define UNIX_H

#ifndef _LARGEFILE64_SOURCE
#define _LARGEFILE64_SOURCE
#endif
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/statfs.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <errno.h>
#include <fcntl.h>
#include <utime.h>

#include "zoid_api.h"

#ifndef __set_errno
#define __set_errno(n) errno = (n)
#endif

#define unix_decode_result(result) \
({ \
    typeof (result) _result = (result); \
    int _n; \
    if ((_n = __zoid_error()) != 0) \
    { \
	__set_errno(_n); \
	_result = -1; \
    } \
    else if (_result < 0) \
    { \
	__set_errno(-_result); \
	_result = -1; \
    } \
    _result; \
})

/* START-ZOID-SCANNER ID=2 INIT=unix__init FINI=unix__fini */

int unix_open(const char *pathname /* in:str */,
	      int flags /* in:obj */,
	      mode_t mode /* in:obj */);

int unix_close(int fd /* in:obj */);

int unix_dup(int oldfd /* in:obj */);

int unix_dup2(int oldfd /* in:obj */,
	      int newfd /* in:obj */);

ssize_t unix_read(int fd /* in:obj */,
		  void *buf /* out:arr:size=+1:zerocopy */,
		  size_t *count /* inout:ptr */);

ssize_t unix_write(int fd /* in:obj */,
		   const void *buf /* in:arr:size=+1:zerocopy */,
		   size_t count /* in:obj */);

/* Not supported by BLRTS.  */
ssize_t unix_pread(int fd /* in:obj */,
		   void *buf /* out:arr:size=+1:zerocopy */,
		   size_t *count /* inout:ptr */,
		   off_t offset /* in:obj */);

/* Not supported by BLRTS.  */
ssize_t unix_pwrite(int fd /* in:obj */,
		    const void *buf /* in:arr:size=+1:zerocopy */,
		    size_t count /* in:obj */,
		    off_t offset /* in:obj */);

/* Not supported by BLRTS.  */
ssize_t unix_pread64(int fd /* in:obj */,
		     void *buf /* out:arr:size=+1:zerocopy */,
		     size_t *count /* inout:ptr */,
		     off64_t offset /* in:obj */);

/* Not supported by BLRTS.  */
ssize_t unix_pwrite64(int fd /* in:obj */,
		      const void *buf /* in:arr:size=+1:zerocopy */,
		      size_t count /* in:obj */,
		      off64_t offset /* in:obj */);

ssize_t unix_readv(int fd /* in:obj */,
		   void** bases /* out:arr2d:size=+1:zerocopy */,
		   size_t* lens /* inout:arr:size=+1 */,
		   int count /* in:obj */);

ssize_t unix_writev(int fd /* in:obj */,
		    const void** bases /* in:arr2d:size=+1:zerocopy */,
		    size_t* lens /* in:arr:size=+1 */,
		    int count /* in:obj */);

off_t unix_lseek(int fildes /* in:obj */,
		 off_t offset /* in:obj */,
		 int whence /* in:obj */);

off64_t unix_lseek64(int fildes /* in:obj */,
		     off64_t offset /* in:obj */,
		     int whence /* in:obj */);

int unix_access(const char *pathname /* in:str */,
		int mode /* in:obj */);

int unix_chmod(const char *path /* in:str */,
	       mode_t mode /* in:obj */);

int unix_fchmod(int fildes /* in:obj */,
		mode_t mode /* in:obj */);

int unix_chown(const char *path /* in:str */,
	       uid_t owner /* in:obj */,
	       gid_t group /* in:obj */);

int unix_fchown(int fd /* in:obj */,
		uid_t owner /* in:obj */,
		gid_t group /* in:obj */);

int unix_lchown(const char *path /* in:str */,
		uid_t owner /* in:obj */,
		gid_t group /* in:obj */);

int unix_stat(const char *path /* in:str */,
	      mode_t *st_mode /* out:ptr */,
	      ino_t *st_ino /* out:ptr */,
	      dev_t *st_dev /* out:ptr */,
	      nlink_t *st_nlink /* out:ptr */,
	      uid_t *st_uid /* out:ptr */,
	      gid_t *st_gid /* out:ptr */,
	      off_t *st_size /* out:ptr */,
	      time_t *stat_atime /* out:ptr */,
	      time_t *stat_mtime /* out:ptr */,
	      time_t *stat_ctime /* out:ptr */,
	      dev_t *st_rdev /* out:ptr */,
	      __blksize_t *st_blksize /* out:ptr */,
	      blkcnt_t *st_blocks /* out:ptr */);

int unix_fstat(int filedes /* in:obj */,
	       mode_t *st_mode /* out:ptr */,
	       ino_t *st_ino /* out:ptr */,
	       dev_t *st_dev /* out:ptr */,
	       nlink_t *st_nlink /* out:ptr */,
	       uid_t *st_uid /* out:ptr */,
	       gid_t *st_gid /* out:ptr */,
	       off_t *st_size /* out:ptr */,
	       time_t *stat_atime /* out:ptr */,
	       time_t *stat_mtime /* out:ptr */,
	       time_t *stat_ctime /* out:ptr */,
	       dev_t *st_rdev /* out:ptr */,
	       __blksize_t *st_blksize /* out:ptr */,
	       blkcnt_t *st_blocks /* out:ptr */);

int unix_lstat(const char *path /* in:str */,
	       mode_t *st_mode /* out:ptr */,
	       ino_t *st_ino /* out:ptr */,
	       dev_t *st_dev /* out:ptr */,
	       nlink_t *st_nlink /* out:ptr */,
	       uid_t *st_uid /* out:ptr */,
	       gid_t *st_gid /* out:ptr */,
	       off_t *st_size /* out:ptr */,
	       time_t *stat_atime /* out:ptr */,
	       time_t *stat_mtime /* out:ptr */,
	       time_t *stat_ctime /* out:ptr */,
	       dev_t *st_rdev /* out:ptr */,
	       __blksize_t *st_blksize /* out:ptr */,
	       blkcnt_t *st_blocks /* out:ptr */);

int unix_stat64(const char *path /* in:str */,
		mode_t *st_mode /* out:ptr */,
		ino64_t *st_ino /* out:ptr */,
		dev_t *st_dev /* out:ptr */,
		nlink_t *st_nlink /* out:ptr */,
		uid_t *st_uid /* out:ptr */,
		gid_t *st_gid /* out:ptr */,
		off64_t *st_size /* out:ptr */,
		time_t *stat_atime /* out:ptr */,
		time_t *stat_mtime /* out:ptr */,
		time_t *stat_ctime /* out:ptr */,
		dev_t *st_rdev /* out:ptr */,
		__blksize_t *st_blksize /* out:ptr */,
		blkcnt64_t *st_blocks /* out:ptr */);

int unix_fstat64(int filedes /* in:obj */,
		 mode_t *st_mode /* out:ptr */,
		 ino64_t *st_ino /* out:ptr */,
		 dev_t *st_dev /* out:ptr */,
		 nlink_t *st_nlink /* out:ptr */,
		 uid_t *st_uid /* out:ptr */,
		 gid_t *st_gid /* out:ptr */,
		 off64_t *st_size /* out:ptr */,
		 time_t *stat_atime /* out:ptr */,
		 time_t *stat_mtime /* out:ptr */,
		 time_t *stat_ctime /* out:ptr */,
		 dev_t *st_rdev /* out:ptr */,
		 __blksize_t *st_blksize /* out:ptr */,
		 blkcnt64_t *st_blocks /* out:ptr */);

int unix_lstat64(const char *path /* in:str */,
		 mode_t *st_mode /* out:ptr */,
		 ino64_t *st_ino /* out:ptr */,
		 dev_t *st_dev /* out:ptr */,
		 nlink_t *st_nlink /* out:ptr */,
		 uid_t *st_uid /* out:ptr */,
		 gid_t *st_gid /* out:ptr */,
		 off64_t *st_size /* out:ptr */,
		 time_t *stat_atime /* out:ptr */,
		 time_t *stat_mtime /* out:ptr */,
		 time_t *stat_ctime /* out:ptr */,
		 dev_t *st_rdev /* out:ptr */,
		 __blksize_t *st_blksize /* out:ptr */,
		 blkcnt64_t *st_blocks /* out:ptr */);

int unix_utime(const char *filename /* in:str */,
	       const struct utimbuf *buf /* in:ptr:nullok */);

ssize_t unix_getdents(int fd /* in:obj */,
		      char *buf /* out:arr:size=+1 */,
		      size_t nbytes /* in:obj */);

ssize_t unix_getdents64(int fd /* in:obj */,
			char *buf /* out:arr:size=+1 */,
			size_t nbytes /* in:obj */);

int unix_chdir(const char *path /* in:str */);

/* Not supported by BLRTS.  */
int unix_fchdir(int fd /* in:obj */);

int unix_mkdir(const char *pathname /* in:str */,
	       mode_t mode /* in:obj */);

int unix_rmdir(const char *pathname /* in:str */);

int unix_getcwd(char *buf /* out:arr:size=+1 */,
		size_t size /* in:obj */);

mode_t unix_umask(mode_t mask /* in:obj */);

int unix_fsync(int fd /* in:obj */);

int unix_fcntl(int fd /* in:obj */,
	       int cmd /* in:obj */,
	       void *arg /* inout:arr:size=+1:nullok */,
	       int *arglen /* inout:ptr */);

int unix_truncate(const char *path /* in:str */,
		  off_t length /* in:obj */);

/* Not supported by BLRTS.  */
int unix_truncate64(const char *path /* in:str */,
		    off64_t length /* in:obj */);

int unix_ftruncate(int fd /* in:obj */,
		   off_t length /* in:obj */);

int unix_ftruncate64(int fd /* in:obj */,
		     off64_t length /* in:obj */);

int unix_link(const char *oldpath /* in:str */,
	      const char *newpath /* in:str */);

int unix_symlink(const char *oldpath /* in:str */,
		 const char *newpath /* in:str */);

int unix_unlink(const char *pathname /* in:str */);

int unix_readlink(const char *path /* in:str */,
		  char *buf /* out:arr:size=+1 */,
		  size_t bufsiz /* in:obj */);

int unix_rename(const char *oldpath /* in:str */,
		const char *newpath /* in:str */);

int unix_socket(int domain /* in:obj */,
		int type /* in:obj */,
		int protocol /* in:obj */);

int unix_connect(int sockfd /* in:obj */,
		 void *serv_addr /* in:arr:size=+1 */,
		 socklen_t addrlen /* in:obj */);

ssize_t unix_send(int s /* in:obj */,
		  const void *buf /* in:arr:size=+1:zerocopy */,
		  size_t len /* in:obj */,
		  int flags /* in:obj */);

ssize_t unix_recv(int s /* in:obj */,
		  void *buf /* out:arr:size=+1:zerocopy */,
		  size_t *len /* inout:ptr */,
		  int flags /* in:obj */);

ssize_t unix_sendto(int s /* in:obj */,
		    const void *buf /* in:arr:size=+1:zerocopy */,
		    size_t len /* in:obj */,
		    int flags /* in:obj */,
		    const void *to /* in:arr:size=+1:nullok */,
		    socklen_t tolen /* in:obj */);

ssize_t unix_recvfrom(int s /* in:obj */,
		      void *buf /* out:arr:size=+1:zerocopy */,
		      size_t *len /* inout:ptr */,
		      int flags /* in:obj */,
		      void *from /* out:arr:size=+1:nullok */,
		      socklen_t *fromlen /* inout:ptr */);

int unix_getsockname(int s /* in:obj */,
		     void *name /* out:arr:size=+1 */,
		     socklen_t *namelen /* inout:ptr */);

int unix_getpeername(int s /* in:obj */,
		     void *name /* out:arr:size=+1 */,
		     socklen_t *namelen /* inout:ptr */);

/* Not supported by BLRTS.  */
int unix_getsockopt(int s /* in:obj */,
		    int level /* in:obj */,
		    int optname /* in:obj */,
		    void *optval /* out:arr:size=+1:nullok */,
		    socklen_t *optlen /* inout:ptr */);

/* Not supported by BLRTS.  */
int unix_setsockopt(int s /* in:obj */,
		    int level /* in:obj */,
		    int optname /* in:obj */,
		    const void *optval /* in:arr:size=+1:nullok */,
		    socklen_t optlen /* in:obj */);

/* The below were added in V1R3M1.  */
int unix_statfs(const char *file /* in:str */,
		struct statfs *buf /* out:ptr */);

int unix_fstatfs(int fd /* in:obj */,
		 struct statfs *buf /* out:ptr */);

int unix_statfs64(const char *file /* in:str */,
		  struct statfs64 *buf /* out:ptr */);

int unix_fstatfs64(int fd /* in:obj */,
		   struct statfs64 *buf /* out:ptr */);

#endif /* UNIX_H */
