
#ifndef ZINT_HANDLER_H
#define ZINT_HANDLER_H

#include "zoidfs.h"

/**
 * NOTE: local should always be last as it will resolve paths successfully
 * as long as the path exists (even if its a mountpoint for a different fs).
 */
typedef enum
{
    ZINT_HANDLE_TYPE_PVFS2 = 1,
    ZINT_HANDLE_TYPE_LOCAL = 2
} zint_handle_type_t;

#define ZOIDFS_HANDLE_TYPE(_handle) \
    ((*(uint32_t *)((uint8_t *)(_handle) + 12)) & 0xFFFFFFFF)

typedef void (* zint_null_handler_t)(void);

typedef int (* zint_getattr_handler_t)(const zoidfs_handle_t * handle,
                                       zoidfs_attr_t * attr);

typedef int (* zint_setattr_handler_t)(const zoidfs_handle_t * handle,
                                       const zoidfs_sattr_t * sattr,
                                       zoidfs_attr_t * attr);

typedef int (* zint_lookup_handler_t)(const zoidfs_handle_t * parent_handle,
                                      const char * component_name,
                                      const char * full_path,
                                      zoidfs_handle_t * handle);

typedef int (* zint_readlink_handler_t)(const zoidfs_handle_t * handle,
                                        char * buffer,
                                        size_t buffer_length);

typedef int (* zint_read_handler_t)(const zoidfs_handle_t * handle,
                                    int mem_count,
                                    void * mem_starts[],
                                    const size_t mem_sizes[],
                                    int file_count,
                                    const uint64_t file_starts[],
                                    uint64_t file_sizes[]);

typedef int (* zint_write_handler_t)(const zoidfs_handle_t * handle,
                                     int mem_count,
                                     const void * mem_starts[],
                                     const size_t mem_sizes[],
                                     int file_count,
                                     const uint64_t file_starts[],
                                     uint64_t file_sizes[]);

typedef int (* zint_commit_handler_t)(const zoidfs_handle_t * handle);

typedef int (* zint_create_handler_t)(const zoidfs_handle_t * parent_handle,
                                      const char * component_name,
                                      const char * full_path,
                                      const zoidfs_sattr_t * attr,
                                      zoidfs_handle_t * handle,
                                      int * created);

typedef int (* zint_remove_handler_t)(const zoidfs_handle_t * parent_handle,
                                      const char * component_name,
                                      const char * full_path,
                                      zoidfs_cache_hint_t * parent_hint);

typedef int (* zint_rename_handler_t)(
    const zoidfs_handle_t * from_parent_handle,
    const char * from_component_name,
    const char * from_full_path,
    const zoidfs_handle_t * to_parent_handle,
    const char * to_component_name,
    const char * to_full_path,
    zoidfs_cache_hint_t * from_parent_hint,
    zoidfs_cache_hint_t * to_parent_hint);

typedef int (* zint_symlink_handler_t)(
    const zoidfs_handle_t * from_parent_handle,
    const char * from_component_name,
    const char * from_full_path,
    const zoidfs_handle_t * to_parent_handle,
    const char * to_component_name,
    const char * to_full_path,
    const zoidfs_sattr_t * attr,
    zoidfs_cache_hint_t * from_parent_hint,
    zoidfs_cache_hint_t * to_parent_hint);

typedef int (* zint_mkdir_handler_t)(const zoidfs_handle_t * parent_handle,
                                     const char * component_name,
                                     const char * full_path,
                                     const zoidfs_sattr_t * attr,
                                     zoidfs_cache_hint_t * parent_hint);

typedef int (* zint_readdir_handler_t)(const zoidfs_handle_t * parent_handle,
                                       zoidfs_dirent_cookie_t cookie,
                                       int * entry_count,
                                       zoidfs_dirent_t * entries,
                                       zoidfs_cache_hint_t * parent_hint);

typedef int (* zint_resize_handler_t)(const zoidfs_handle_t * handle,
                                      uint64_t size);

typedef int (* zint_resolve_path_handler_t)(const char * local_path,
                                            zoidfs_handle_t * handle,
                                            char * fs_path,
                                            int fs_path_max);

typedef int (* zint_init_t)(void);

typedef int (* zint_finalize_t)(void);

typedef struct
{
    zint_handle_type_t type;
    zint_null_handler_t null;
    zint_getattr_handler_t getattr;
    zint_setattr_handler_t setattr;
    zint_lookup_handler_t lookup;
    zint_readlink_handler_t readlink;
    zint_read_handler_t read;
    zint_write_handler_t write;
    zint_commit_handler_t commit;
    zint_create_handler_t create;
    zint_remove_handler_t remove;
    zint_rename_handler_t rename;
    zint_symlink_handler_t symlink;
    zint_mkdir_handler_t mkdir;
    zint_readdir_handler_t readdir;
    zint_resize_handler_t resize;
    zint_resolve_path_handler_t resolve_path;
    zint_init_t init;
    zint_finalize_t finalize;
} zint_handler_t;

#endif /* ZINT_HANDLER_H */

/*
 * Local variables:
 *  c-indent-level: 4
 *  c-basic-offset: 4
 * End: 
 *
 * vim: ts=8 sts=4 sw=4 expandtab
 */
